/* 
   File: cholesky.h
   
   Class that performs the Cholesky decomposition. 
*/

#include <iostream>
#include <vector>
#include <math.h>
#include <assert.h>
#include "linalg.h"
#include "solvers.h"

#ifndef CHOLESKY_H
#define CHOLESKY_H

using namespace std;

typedef vector< vector<double> > DoubMat;

/*
  Class: Cholesky

  Performs the Cholesky decomposition and performs matrix operations using the decomposed matrix. 
*/
class Cholesky{ 
  // Object for Cholesky decomposition of a matrix A, and related functions.
  int n;
  alglib::real_2d_array L; // Stores the decomposition.

 public: 
  /* 
     Constructor: Cholesky

     Constructor. Given a positive-definite symmetric matrix a[0..n-1][0..n-1], construct
     and store its Cholesky decomposition, A = L . L^T 

   */
  Cholesky(DoubMat &a) : n((int)a.size()) {
    
    assert(a[0].size() == n);

    // copy data 
    L.setlength(n,n);
    for(int i = 0; i < n; i++) {
      for(int j = 0; j < n; j++) {
	L[i][j] = a[i][j];	
      }     
    }   

    double det = alglib::rmatrixdet(L);
    cout << "determinant of cov: " << det << endl;

    // computer decomp
    if(! alglib::spdmatrixcholesky(L, n,true) ) {
      throw "non-spd matrix";
    }
  }


  // Function: logdet
  // Return the logarithm of the determinant of A, the matrix whose Cholesky decomposition
  //  has been stored.
  // Returns:
  // The determinant.
  double logdet() {
    return log( alglib::spdmatrixcholeskydet(L));
  }

  // Function: Lsolve
  // Solve L . y = b, where L is the lower triangular matrix in the stored Cholesky decomposition.
  // Parameters:
  // b - right-hand side of linear equations
  // y - solution to linear equations
  // Returns:
  // y holds solutions. 
  void Lsolve(vector<double> &b, vector<double> &y) {

    assert(b.size() == n && y.size() == n); // input vectors are improper size
    assert(L.rows() == n && L.cols() == n); // L is improper size

    alglib::ae_int_t info;
    alglib::densesolverreport rep;
    alglib::real_1d_array x,outy;


    // copy input
    x.setlength(b.size());
    for(int i = 0; i < b.size(); i++) x[i] = b[i];

    // solve
    alglib::spdmatrixcholeskysolve(L,n,true,x,info,rep,outy);

    // copy output
    if(info == 1) { 
      for(int i = 0; i < n; i++) { 
	y[i] = outy[i];
      }
    } else {
      cout << "Error with cholesky: info = " << info << endl;
      if(info == -3) cout << "cov. mat. is (near) singular" << endl;
      else cout << "cov. mat. is wrong size" << endl;
    }
  }
};

#endif
