/*
  File: sim.cpp

  Contains the example code of using the GMM model. 

  Dependences:

  <gauss_model.h>
  

  Program: sim

  This program fits a Gaussian Mixture Model (GMM) to the provided data. The
  GMM is trained using iterations of the Expectation-Maximization (EM) method.
  The GMM results in posterior probabilities for each data sample for all 
  mixtures in the model. A heuristic is then applied to assign each data 
  sample to one or more mixtures. The results are stored in three files. 

  Usage: sim <em_iters> <num_clusters> <cov type> <init type> <data file> <name file> [model file]

  Parameters:

  em_iters - number of EM iterations to perform
  num_clusters - number of mixtures/clusters 
  cov type - type of covariance matrix (see gauss_model.h)
  init type - method of initializing the model parameters
  data file - file that has data sample
  name file - file that has the name of all data samples
  model file - optional: file that holds GMM to use for initial values. 


*/
#include <iostream>
#include <time.h>
#include "gauss_model.h"

using namespace std;

/*
  Function: main

  Instantiates the GMM Model, which produces posterior probabilities, and calls
  the clustering function to calculate assignments of the data samples. 

  Parameters:

  argc - number of arguments passed to the program
  argv - arguments passed to the program
*/
int main(int argc, char* argv[]) {

  if( argc < 7) {
    std::cout << "usage: sim <em_iters> <# of clusters> <cov_type> <init type>  <data file> <name file> [model file] <scale_parameter> " << std::endl;
    std::cout << "data file starts with: <# samps> <# dims>" << std::endl;
    std::cout << "inits: 0 - randomly choose k (k = num mixes) " << endl;
    std::cout << "inits: 1 - greedly choose k (k = num mixes) " << endl;
    std::cout << "inits: 2 - randomly choose k log k and merge (k = num mixes) " << endl;
    std::cout << "inits: 3 - load model from a file" << endl;
    //exit(0);
    return 1; 
  }
  
  //system("date");
  for(int i = 0; i < argc; i++) {
    cout << argv[i] << " ";
  }
  cout << endl;

  unsigned int sseed = time(NULL);
  sseed = 1298582040; // uncomment and use this line to repeat a specific experiment
  cout << "Seed: " << sseed << endl;
  srand(sseed);

  int em_iters = atoi(argv[1]);
  int num_mixes = atoi(argv[2]);
  int cov_type = atoi(argv[3]);
  int init_type = atoi(argv[4]);
  double scale_parameter =0.01 ;

  if(init_type == 3 && argc < 8) { 
    cout << "Input error: No model file given to load." << endl;
    //exit(0);
    return 1;
  }
  if(init_type!=3) {
	if(argc==8) {
		scale_parameter = atof(argv[7]);
	}
  }

 cout << "Scale parameter "<< scale_parameter <<endl ;

  Gauss_model *gmm_model;
  
  if(init_type < 3 ) { 
    gmm_model = new Gauss_model(argv[5], num_mixes, cov_type, init_type);
  } else { 
    gmm_model = new Gauss_model(argv[5], num_mixes, cov_type, init_type, argv[7]);
  }

  
  clock_t t1,t2;
  t1 = clock();

  printf("Initial Log like. = %f\n", gmm_model->loglike);

  // perform subsequent iterations
  for(int i = 0; i < em_iters; i++ ) {
    printf("Performing step %d \n",i);
    gmm_model->estep();
    printf("Log like. = %f\n", gmm_model->loglike);
    gmm_model->mstep(scale_parameter);
  }
  t2 = clock();
  gmm_model->estep();
  printf( "Final Log like. = %f\n",gmm_model->loglike); 

  printf( "Training time: %f seconds\n",double(t2-t1)/CLOCKS_PER_SEC);

  // output model, clustering, and posterior probabilities to files
  gmm_model->PrintModel("model.txt");
  gmm_model->PrintClusterAssignments("cluster_assigns.txt");
  gmm_model->PrintProbs("probs.txt");
  gmm_model->PrintSortedProbs("sorted_probs.txt",argv[6]);

  cout << "Finished"<< endl;
  return 0;
}


